import pandas as pd
from pysent3.base import STATIC_PATH, BaseDict


class LM(BaseDict):
    """
    Dictionary class for
    Loughran and McDonald Financial Sentiment Dictionaries.
    
    See also https://www3.nd.edu/~mcdonald/Word_Lists.html
    
    The terms for the dictionary are stemmed by the default tokenizer.
    """
    
    PATH = '%s/LM.csv' % STATIC_PATH
    
    def init_dict(self):
        data = pd.read_csv(self.PATH)
        self._posset = set(data.query('Positive > 0')['Word'].apply(self.tokenize_first).dropna())
        self._negset = set(data.query('Negative > 0')['Word'].apply(self.tokenize_first).dropna())



import pandas as pd
from pysent3.base import STATIC_PATH, BaseDict


class HIV4(BaseDict):
    """
    Dictionary class for Harvard IV-4. 
    See also http://www.wjh.harvard.edu/~inquirer/
    
    The terms for the dictionary are stemmed by the default tokenizer.
    """
    
    PATH = '%s/HIV-4.csv' % STATIC_PATH
    
    def init_dict(self):
        data = pd.read_csv(self.PATH, dtype='category')
        self._posset = set(data.query('Positiv == "Positiv"')['Entry'].apply(self.tokenize_first).dropna())
        self._negset = set(data.query('Negativ == "Negativ"')['Entry'].apply(self.tokenize_first).dropna())



import unittest
from pysent3.utils import Tokenizer


class TestTokenizer(unittest.TestCase):

    def test_token(self):
        text = 'this is a wonderful gift.'
        tokenizer = Tokenizer()
        self.assertTrue(tokenizer.tokenize(text)==['wonder', 'gift'])


if __name__ == "__main__":
    unittest.main()



"""
This module contains base classes for dictionaries.
"""

import abc
import os
import numpy as np
from pysent3.utils import Tokenizer

STATIC_PATH = os.path.join(os.path.dirname(__file__), 'static')


def negated(word):
    from pysent3.base import STATIC_PATH
    text_file = open('%s/%s' % (STATIC_PATH, 'NegationWords.txt'), 'rb')
    negate = text_file.read().decode("utf-8").splitlines()
    negate = [w.lower() for w in negate]
    if word.lower() in negate:
        return True
    else:
        return False


class BaseDict(object):
    """
    A base class for sentiment analysis. 
    For now, only 'positive' and 'negative' analysis is supported.
    
    Subclasses should implement ``init_dict``, 
    in which ``_posset`` and ``_negset`` are initialized.
    
    ``Polarity`` and ``Subjectivity`` are calculated in the same way of Lydia system.
    See also http://www.cs.sunysb.edu/~skiena/lydia/
    
    The formula for ``Polarity`` is,
    
    .. math::
    
        Polarity= \\frac{N_{pos}-N_{neg}}{N_{pos}+N_{neg}}
    
    The formula for ``Subjectivity`` is,
    
    .. math::
    
        Subjectivity= \\frac{N_{pos}+N_{neg}}{N}
    
    :type tokenizer: obj    
    :param tokenizer: An object which provides interface of ``tokenize``. 
        If it is ``None``, a default tokenizer, which is defined in ``utils``, will be assigned.
    """

    __metaclass__ = abc.ABCMeta

    TAG_POL = 'Polarity'
    TAG_SUB = 'Subjectivity'
    TAG_POS = 'Positive'
    TAG_NEG = 'Negative'

    EPSILON = 1e-6

    def __init__(self, tokenizer=None):
        self._posset = set()
        self._negset = set()
        if tokenizer is None:
            self._tokenizer = Tokenizer()
        else:
            self._tokenizer = tokenizer
        self.init_dict()

        assert len(self._posset) > 0 and len(self._negset) > 0

    def tokenize(self, text):
        """
        :type text: str
        :returns: list
        """

        return self._tokenizer.tokenize(text)

    def tokenize_first(self, x):
        """
        :type x: str
        :returns: str
        """
        tokens = self.tokenize(x)
        if tokens:
            return tokens[0]
        else:
            return None

    @abc.abstractmethod
    def init_dict(self):
        pass

    def _get_score(self, terms):
        """Get scores for each term.

        - +1 for positive terms.
        - -1 for negative terms.
        - 0 for others. 
        
        :returns: list
        """
        scores = []

        word_count = len(terms)

        for i in range(0, word_count):
            if terms[i] in self._negset:
                if i >= 3:
                    if negated(terms[i - 1]) or negated(terms[i - 2]) or negated(terms[i - 3]):
                        scores.append(1)
                    else:
                        scores.append(-1)
                elif i == 2:
                    if negated(terms[i - 1]) or negated(terms[i - 2]):
                        scores.append(1)
                    else:
                        scores.append(-1)
                elif i == 1:
                    if negated(terms[i - 1]):
                        scores.append(1)
                    else:
                        scores.append(-1)
                elif i == 0:
                    scores.append(-1)
            elif terms[i] in self._posset:
                if i >= 3:
                    if negated(terms[i - 1]) or negated(terms[i - 2]) or negated(terms[i - 3]):
                        scores.append(-1)
                    else:
                        scores.append(1)
                elif i == 2:
                    if negated(terms[i - 1]) or negated(terms[i - 2]):
                        scores.append(-1)
                    else:
                        scores.append(1)
                elif i == 1:
                    if negated(terms[i - 1]):
                        scores.append(-1)
                    else:
                        scores.append(1)
                elif i == 0:
                    scores.append(1)
            else:
                scores.append(0)

        return scores

    def get_score(self, terms):
        """Get score for a list of terms.
        
        :type terms: list
        :param terms: A list of terms to be analyzed.
        
        :returns: dict
        """
        assert isinstance(terms, list) or isinstance(terms, tuple)
        score_li = np.asarray(self._get_score(terms))
        # score_li = np.asarray([self._get_score(t) for t in terms])
        # print(score_li)

        s_pos = np.sum(score_li[score_li > 0])
        s_neg = -np.sum(score_li[score_li < 0])

        s_pol = (s_pos - s_neg) * 1.0 / ((s_pos + s_neg) + self.EPSILON)
        s_sub = (s_pos + s_neg) * 1.0 / (len(score_li) + self.EPSILON)

        return {self.TAG_POS: s_pos,
                self.TAG_NEG: s_neg,
                self.TAG_POL: s_pol,
                self.TAG_SUB: s_sub}



import unittest
import numpy as np
from pysent3.hiv4 import HIV4
from pysent3.lm import LM


class TestDict(unittest.TestCase):

    def setUp(self):
        self.text = '''The Pandora papers represent the latest – and largest in terms of data volume – in a series of 
        major leaks of financial data that have convulsed the offshore world since 2013. Setting up or benefiting from 
        offshore entities is not itself illegal, and in some cases people may have legitimate reasons, such as security,
        for doing so. But the secrecy offered by tax havens has at times proven attractive to tax evaders, fraudsters
        and money launderers, some of whom are exposed in the files.
        
        The Pandora papers also place a revealing spotlight on the offshore system itself. In a development likely to 
        prove embarrassing for the US president, Joe Biden, who has pledged to lead efforts internationally to bring 
        transparency to the global financial system, the US emerges from the leak as a leading tax haven. The files 
        suggest the state of South Dakota, in particular, is sheltering billions of dollars in wealth linked to 
        individuals previously accused of serious financial crimes.'''

    def test_hiv4(self):
        hiv4 = HIV4()
        tokens = hiv4.tokenize(self.text)
        score = hiv4.get_score(tokens)
        self.assertEqual(score['Positive'], 19)
        self.assertEqual(score['Negative'], 13)
        self.assertTrue(np.isclose(score['Polarity'], 0.1875))
        self.assertTrue(np.isclose(score['Subjectivity'], 0.19999999))

    def test_lm(self):
        lm = LM()
        tokens = lm.tokenize(self.text)
        score = lm.get_score(tokens)
        self.assertEqual(score['Positive'], 6)
        self.assertEqual(score['Negative'], 8)
        self.assertTrue(np.isclose(score['Polarity'], -0.14285714))
        self.assertTrue(np.isclose(score['Subjectivity'], 0.08749999))


if __name__ == "__main__":
    unittest.main()
